
# Installation requirements
require(solaR)
require(hexbin)
require(ggplot2)


# Read data (csv file)
args <- commandArgs(TRUE)
data <- read.csv(args[1])

# Create figures folder if it does not exist
if (file.exists("figures")){
    message ("Figures folder already exists")
} else {
    dir.create("figures")
}

# Removing unwanted columns manually
data <- data[ -c(1:5)]
data <- data[ -c(13:15)]


# Renaming column names
colnames(data)[1] <- "MeanCPU"
colnames(data)[2] <- "CanMemory"
colnames(data)[3] <- "AssignedMem"
colnames(data)[4] <- "UnmappedCache"
colnames(data)[5] <- "TotPageCache"
colnames(data)[6] <- "MaxMem"
colnames(data)[7] <- "MeanIO"
colnames(data)[8] <- "MeanDiskSpace"
colnames(data)[9] <- "MaxCPU"
colnames(data)[10] <- "MaxIOTime"
colnames(data)[11] <- "CPI"
colnames(data)[12] <- "MAI"


# I generate a random sample of the original dataset of 2.5 million rows.
data2=data[sample(nrow(data), replace=F, size=100000),]

# I normalyze the data so there is no NA. I replace it with 0 so that everything is numerical.
data2[is.na(data2)] <- 0

# PCA Analysis
pca <- prcomp(data2, scale=TRUE)

# With covariance matrix
(pca.cov <- prcomp(data2))
diag(1/sqrt(diag(cov(data2)))) %*% pca.cov$rotation %*% diag(pca.cov$sdev)
(corvar2 <- pca.cov$rotation %*% diag(pca.cov$sdev))

# Summary
summary(pca)

# Correlation between variables and components
(corvar <- pca$rotation %*% diag(pca$sdev))

# Representation
# Bar plot of the weight of each component
png(filename="figures/barplot1.png", width = 1024, height = 768)
barplot(summary(pca)$importance[2,])
dev.off()

# Correlation between variables and components PC1 and PC2
png(filename="figures/correlation.png", width = 1000, height = 1000)
plot(-1:1, -1:1, type='n', xlab='PC1', ylab='PC2', xlim = c(-1,1))
abline(h=0, v=0, lty=2, col=8)
#symbols(0, 0, 1, inches=F, add=T)
#symbols(0, 0, sqrt(.5), inches=F, add=T)
arrows(0, 0, corvar[,1], corvar[,2], length=.1)
text(corvar[,1], corvar[,2], colnames(data2), pos=4, offset=.6, col=2, font=2)
dev.off()

# Correlation between variables and components PC1, PC2 and PC3 in a bar plot
png(filename="figures/barplot2.png", width = 1920, height = 1080)
barplot(t(corvar[,1:3]),beside=TRUE, ylim=c(-1,1))
legend("topleft",legend = c("PC1","PC2","PC3"), fill = c("black","darkgray","lightgray"))
dev.off()

# Variables relationship. Requires Hexbin and solaR
png(filename="figures/matrix.png", width = 1400, height = 1400)
splom(data2[1:15000,1:12],
      panel=panel.hexbinplot,
      diag.panel = function(x, ...){
        yrng <- current.panel.limits()$ylim
        d <- density(x, na.rm=TRUE)
        d$y <- with(d, yrng[1] + 0.95 * diff(yrng) * y / max(y) )
        panel.lines(d)
        diag.panel.splom(x, ...)
      },
      lower.panel = function(x, y, ...){
        panel.hexbinplot(x, y, ...)
        panel.loess(x, y, ..., col = 'red')
      },
      pscale=0, varname.cex=1.5
)
dev.off()


# Training and Test. K-Means.
library(flexclust)

Train <- data2[1:8000,]
Test <- data2[8001:8500,]

pca.train <- princomp(Train,cor=TRUE)
pca3.train <- prcomp(Train, scale=TRUE)

png(filename="figures/pca.train.png", width = 1024, height = 768)
plot(pca.train)
dev.off()

pc1.train <- pca.train$scores[,1]
pc2.train <- pca.train$scores[,2]

X.train <- cbind(pc1.train,pc2.train) 

pca.test <- predict(pca.train, newdata=Test)

pc.comp.test <- pca.test[,1:2]
pc1.test <- pc.comp.test[,1]
pc2.test <- pc.comp.test[,2]

X.test <- cbind(pc1.test,pc2.test)

cl <- kcca(X.train,k=3,kccaFamily("kmeans"),control = list(initcent="kmeanspp"))

pred_train <- predict(cl)
pred_test <- predict(cl,newdata=X.test)

pdf(file="figures/k-means.pdf")
image(cl,fastcol=FALSE,graph=TRUE)

image(cl,fastcol=FALSE,graph=TRUE)
points(X.train,col=pred_train,cex=.3)

image(cl,fastcol=FALSE,graph=TRUE)
points(X.train,col=pred_train,cex=.3)
points(X.test,col=pred_test,pch=22,bg="orange")
dev.off()


scores4<-pca.train$scores
ggdata2<-data.frame(scores4, Cluster=cl@cluster)
png(filename="figures/kmeans_cluster_pca.png", width = 1024, height = 768)
ggplot(ggdata2) + geom_point(aes(x=Comp.1, y=Comp.2, color=factor(Cluster)), size=5, shape=20) + stat_ellipse(aes(x=Comp.1,y=Comp.2,fill=factor(Cluster)), geom = "polygon", level=0.95, alpha=0.4) + guides(color=guide_legend("Cluster"), fill=guide_legend("Cluster"))
dev.off()
